@@ -102,6 +102,7 @@ process_line (char     *line,
 	FRCommandRar  *rar_comm = FR_COMMAND_RAR (comm);
 	char         **fields;
 	const char    *name_field;
+	gboolean       encrypted;      /* unused */
 
 	g_return_if_fail (line != NULL);
 
@@ -124,8 +125,10 @@ process_line (char     *line,
 		rar_comm->fdata = fdata = file_data_new ();
 
 		/* read file name. */
-
-		name_field = get_last_field (line, 1);
+	  
+		encrypted = (line[0] == '*') ? TRUE : FALSE;
+		
+		name_field = line + 1;
 
 		if (*name_field == '/') {
 			fdata->full_path = g_strdup (name_field);
@@ -140,22 +143,23 @@ process_line (char     *line,
 		fdata->name = g_strdup (file_name_from_path (fdata->full_path));
 		fdata->path = remove_level_from_path (fdata->full_path);
 		fdata->type = gnome_vfs_mime_type_from_name_or_default (fdata->name, GNOME_VFS_MIME_TYPE_UNKNOWN);
+
 	} else {
 		FileData *fdata;
-
+		
 		fdata = rar_comm->fdata;
 
 		/* read file info. */
 		
-		fields = split_line (line, 5);
-		fdata->size = atol (fields[0]);
-		fdata->modified = mktime_from_string (fields[3], fields[4]); 
-		g_strfreev (fields);
-		
-		if (*fdata->name == 0)
+		fields = split_line (line, 6);
+		if ((*fdata->name == 0) || (fields[5][1] == 'D')) {
 			file_data_free (fdata);
-		else
+		} else {
+			fdata->size = atol (fields[0]);
+			fdata->modified = mktime_from_string (fields[3], fields[4]); 
+			g_strfreev (fields);
 			comm->file_list = g_list_prepend (comm->file_list, fdata);
+		}
 
 		rar_comm->fdata = NULL;
 	}
@@ -164,6 +168,29 @@ process_line (char     *line,
 }
 
 
+static void
+add_password_arg (FRCommand	*comm,
+		  const char	*password,
+		  gboolean	disable_query)
+{
+	if (password != NULL && password[0] != '\0') {
+		char *arg, *e_password;
+		
+		e_password = escape_str (password, "\"");
+		g_assert (e_password != NULL);   /* since password is non-NULL */
+		
+		arg = g_strconcat ("-p\"", e_password, "\"", NULL);
+		
+		fr_process_add_arg (comm->process, arg);
+		
+		g_free (e_password);
+		g_free (arg);
+	} else if (disable_query) {
+		fr_process_add_arg (comm->process, "-p-");
+	}
+}
+
+
 static void
 fr_command_rar_list (FRCommand *comm)
 {
@@ -180,6 +207,10 @@ fr_command_rar_list (FRCommand *comm)
 	fr_process_add_arg (comm->process, "v");
 	fr_process_add_arg (comm->process, "-c-");
 
+	/* Fixme: listing rar archives fails when headers and data
+	          are encrypted (-hp option) without a password. */
+	add_password_arg (comm, NULL, TRUE);
+	
 	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--"); 
 
@@ -220,6 +251,8 @@ fr_command_rar_add (FRCommand     *comm,
 		fr_process_add_arg (comm->process, "-m5"); break;
 	}
 
+	add_password_arg (comm, password, FALSE);
+	
 	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--"); 
 
@@ -281,6 +314,8 @@ fr_command_rar_extract (FRCommand  *comm,
 	if (junk_paths)
 		fr_process_add_arg (comm->process, "-ep");
 	
+	add_password_arg (comm, password, TRUE);
+	
 	/* stop switches scanning */
 	fr_process_add_arg (comm->process, "--"); 
 
@@ -299,6 +334,27 @@ fr_command_rar_extract (FRCommand  *comm,
 }
 
 
+static void
+fr_command_rar_test (FRCommand   *comm,
+		     const char  *password)
+{
+	if (fr_command_rar_have_rar ())
+		fr_process_begin_command (comm->process, "rar");
+	else
+		fr_process_begin_command (comm->process, "unrar");
+	
+	fr_process_add_arg (comm->process, "t");
+	
+	add_password_arg (comm, password, TRUE);
+	
+	/* stop switches scanning */
+	fr_process_add_arg (comm->process, "--");
+	
+	fr_process_add_arg (comm->process, comm->e_filename);
+	fr_process_end_command (comm->process);
+}
+
+
 static void
 fr_command_rar_handle_error (FRCommand   *comm, 
 			     FRProcError *error)
@@ -324,6 +380,7 @@ fr_command_rar_class_init (FRCommandRarClass *class)
 	afc->add          = fr_command_rar_add;
 	afc->delete       = fr_command_rar_delete;
 	afc->extract      = fr_command_rar_extract;
+	afc->test         = fr_command_rar_test;
 	afc->handle_error = fr_command_rar_handle_error;
 }
 
@@ -336,8 +393,8 @@ fr_command_rar_init (FRCommand *comm)
 	comm->propExtractCanAvoidOverwrite = TRUE;
 	comm->propExtractCanSkipOlder      = TRUE;
 	comm->propExtractCanJunkPaths      = TRUE;
-	comm->propPassword                 = FALSE;
-	comm->propTest                     = FALSE;
+	comm->propPassword                 = TRUE;
+	comm->propTest                     = TRUE;
 }
 
 
